import { OpenAI } from 'openai';
import type { BaseChatInterface, ChatMessageComplete, ChatMessage, ChatMessageOptions, BaseChatUsageInterface } from './type.ts';

export type BaseChatOptions<T = Record<string, any>> = {
  /**
   * 默认baseURL
   */
  baseURL: string;
  /**
   * 默认模型
   */
  model: string;
  /**
   * 默认apiKey
   */
  apiKey: string;
  /**
   * 是否在浏览器中使用
   */
  isBrowser?: boolean;
  /**
   * 是否流式输出， 默认 false
   */
  stream?: boolean;
} & T;

export class BaseChat implements BaseChatInterface, BaseChatUsageInterface {
  /**
   * 默认baseURL
   */
  baseURL: string;
  /**
   * 默认模型
   */
  model: string;
  /**
   * 默认apiKey
   */
  apiKey: string;
  /**
   * 是否在浏览器中使用
   */
  isBrowser: boolean;
  /**
   * openai实例
   */
  openai: OpenAI;

  prompt_tokens: number;
  total_tokens: number;
  completion_tokens: number;

  constructor(options: BaseChatOptions) {
    this.baseURL = options.baseURL;
    this.model = options.model;
    this.apiKey = options.apiKey;
    this.isBrowser = options.isBrowser ?? false;
    this.openai = new OpenAI({
      apiKey: this.apiKey,
      baseURL: this.baseURL,
      dangerouslyAllowBrowser: this.isBrowser,
    });
  }
  /**
   * 聊天
   */
  async chat(messages: ChatMessage[], options?: ChatMessageOptions): Promise<ChatMessageComplete> {
    const createParams: OpenAI.Chat.Completions.ChatCompletionCreateParams = {
      model: this.model,
      messages,
      ...options,
      stream: false,
    };
    const res = (await this.openai.chat.completions.create(createParams)) as ChatMessageComplete;
    this.prompt_tokens = res.usage?.prompt_tokens ?? 0;
    this.total_tokens = res.usage?.total_tokens ?? 0;
    this.completion_tokens = res.usage?.completion_tokens ?? 0;
    return res;
  }
  async chatStream(messages: ChatMessage[], options?: ChatMessageOptions) {
    const createParams: OpenAI.Chat.Completions.ChatCompletionCreateParams = {
      model: this.model,
      messages,
      ...options,
      stream: true,
    };
    if (createParams.response_format) {
      throw new Error('response_format is not supported in stream mode');
    }
    return this.openai.chat.completions.create(createParams) as any;
  }

  /**
   * 测试
   */
  test() {
    return this.chat([{ role: 'user', content: 'Hello, world!' }]);
  }
  /**
   * 获取聊天使用情况
   * @returns
   */
  getChatUsage() {
    return {
      prompt_tokens: this.prompt_tokens,
      total_tokens: this.total_tokens,
      completion_tokens: this.completion_tokens,
    };
  }
}
